//
//  MyScene.m
//  Flappy Felipe
//
//  Created by Main Account on 2/13/14.
//  Copyright (c) 2014 Razeware LLC. All rights reserved.
//

#import "MyScene.h"

typedef NS_ENUM(int, Layer) {
  LayerBackground,
  LayerForeground,
  LayerPlayer
};

typedef NS_OPTIONS(int, EntityCategory) {
  EntityCategoryPlayer = 1 << 0,
  EntityCategoryObstacle = 1 << 1,
  EntityCategoryGround = 1 << 2
};

// Gameplay - bird movement
static const float kGravity = -1500.0;
static const float kImpulse = 400.0;
static const float kAngularVelocity = 1000;

// Gameplay - max tap frequency
static const float kTapDelay = 0.15;

// Misc
static const float kMinDegrees = -90;
static const float kMaxDegrees = 25;
static const int kNumForegrounds = 2;

@implementation MyScene {

  SKNode *_worldNode;
  
  float _playableStart;
  float _playableHeight;

  NSTimeInterval _lastUpdateTime;
  NSTimeInterval _dt;

  SKSpriteNode *_player;
  CGPoint _playerVelocity;
  
  SKAction * _dingAction;
  SKAction * _flapAction;
  SKAction * _whackAction;
  SKAction * _fallingAction;
  SKAction * _hitGroundAction;
  SKAction * _popAction;
  SKAction * _coinAction;
  
}

-(id)initWithSize:(CGSize)size {
  if (self = [super initWithSize:size]) {
    
    _worldNode = [SKNode node];
    [self addChild:_worldNode];
    
    [self setupBackground];
    [self setupForeground];
    [self setupPlayer];
    [self setupSounds];
  
  }
  return self;
}

#pragma mark - Setup methods

- (void)setupBackground {
  SKSpriteNode *background = [SKSpriteNode spriteNodeWithImageNamed:@"Background"];
  background.anchorPoint = CGPointMake(0.5, 1);
  background.position = CGPointMake(self.size.width/2, self.size.height);
  background.zPosition = LayerBackground;
  [_worldNode addChild:background];
  
  _playableStart = self.size.height - background.size.height;
  _playableHeight = background.size.height;

}

- (void)setupForeground {
  SKSpriteNode *foreground = [SKSpriteNode spriteNodeWithImageNamed:@"Ground"];
  foreground.anchorPoint = CGPointMake(0, 1);
  foreground.position = CGPointMake(0, _playableStart);
  foreground.name = @"Foreground";
  foreground.zPosition = LayerForeground;
  [_worldNode addChild:foreground];
}

- (void)setupPlayer {

  _player = [SKSpriteNode spriteNodeWithImageNamed:@"Bird0"];
  _player.position = CGPointMake(self.size.width * 0.2, _playableHeight * 0.4 + _playableStart);
  _player.zPosition = LayerPlayer;
  [_worldNode addChild:_player];
  
}

- (void)setupSounds {
  _dingAction = [SKAction playSoundFileNamed:@"ding.wav" waitForCompletion:NO];
  _flapAction = [SKAction playSoundFileNamed:@"flapping.wav" waitForCompletion:NO];
  _whackAction = [SKAction playSoundFileNamed:@"whack.wav" waitForCompletion:NO];
  _fallingAction = [SKAction playSoundFileNamed:@"falling.wav" waitForCompletion:NO];
  _hitGroundAction = [SKAction playSoundFileNamed:@"hitGround.wav" waitForCompletion:NO];
  _popAction = [SKAction playSoundFileNamed:@"pop.wav" waitForCompletion:NO];
  _coinAction = [SKAction playSoundFileNamed:@"coin.wav" waitForCompletion:NO];
}

#pragma mark - Gameplay

- (void)flapPlayer {

  // Play sound
  [self runAction:_flapAction];

  // Apply impulse
  _playerVelocity = CGPointMake(0, kImpulse);
  
}

-(void)touchesBegan:(NSSet *)touches withEvent:(UIEvent *)event {
  [self flapPlayer];
}

#pragma mark - Updates

- (void)updatePlayer {
  
  // Apply gravity
  CGPoint gravity = CGPointMake(0, kGravity);
  CGPoint gravityStep = CGPointMultiplyScalar(gravity, _dt);
  _playerVelocity = CGPointAdd(_playerVelocity, gravityStep);

  // Apply velocity
  CGPoint velocityStep = CGPointMultiplyScalar(_playerVelocity, _dt);
  _player.position = CGPointAdd(_player.position, velocityStep);
  
  // Temporary halt when hits ground
  if (_player.position.y - _player.size.height/2 <= _playableStart) {
    _player.position = CGPointMake(_player.position.x, _playableStart + _player.size.height/2);
    return;
  }
  
}

-(void)update:(CFTimeInterval)currentTime {
  if (_lastUpdateTime) {
    _dt = currentTime - _lastUpdateTime;
  } else {
    _dt = 0;
  }
  _lastUpdateTime = currentTime;
  [self updatePlayer];
}

@end
